@@ -61,8 +61,8 @@ def escape_markdown(content: str) -> str:
     :rtype: :obj:`str`
     """
     
-    parse = re.sub(r"([_*\[\]()~`>\#\+\-=|\.!])", r"\\\1", content)
-    reparse = re.sub(r"\\\\([_*\[\]()~`>\#\+\-=|\.!])", r"\1", parse)
+    parse = re.sub(r"([_*\[\]()~`>\#\+\-=|\.!\{\}])", r"\\\1", content)
+    reparse = re.sub(r"\\\\([_*\[\]()~`>\#\+\-=|\.!\{\}])", r"\1", parse)
     return reparse 
 
 
@@ -323,4 +323,4 @@ def hide_link(url: str) -> str:
     :return: The hidden url.
     :rtype: :obj:`str`
     """
-    return f'<a href="{url}">&#8288;</a>'
\ No newline at end of file
+    return f'<a href="{url}">&#8288;</a>'
